#include "main.h"
#include "knightrider.h"
#include "getkey.h"


void init( void )
{
  KEY_PULLUP = 0xFF;
  LED_PORT = 0xFF;				// all LEDs off
  LED_DDR = 0xFF;

  get_key_init();

  PCMSK_REG = 1<<PCINT_BIT;			// prepare awake pin
  set_sleep_mode( SLEEP_MODE_PWR_DOWN);		// prepare sleep
  sleep_enable();
}


uint8_t get_key_busy( uint8_t key_mask )
{
  return (~KEY_PIN | key_state) & key_mask;	// until key released
}


EMPTY_INTERRUPT( PCINT_vect );			// wake up from power down


void try_sleep()
{
  if( get_key_busy( 1<<KEY0 ))
    return;					// still busy
  LED_PORT = 0xFF;				// all LEDs off
  PCICR = 1<<PCICR_ENABLE;			// awake interrupt on
  sleep_cpu();
  PCICR = 0;
}


#define	SPEED		100			// 100ms
#define	TIMEOUT		60000			// 1 min


int main()
{
  uint8_t sample = 1;
  uint8_t stop = 0;
  uint16_t timer = 0;

  select( sample );
  init();
  sei();

  for(;;){
    if( get_key_long( 1<<KEY0 )){		// long press: sleep / awake
      stop ^= 0xFF;
    }
    if( stop ){
      try_sleep();
      LED_PORT = 0x00;				// all LEDs on (for debug only)
      get_key_short( 1<<KEY0 );			// ignore short press during stop
      timer = 0;				// reset timeout on key press
      continue;
    }
    if( get_key_short( 1<<KEY0 )){		// short press: select pattern
      sample++;
      if( select( sample ))
        sample = 1;
      timer = 0;				// reset timeout on key press
    }
    run_pattern();
    _delay_ms( SPEED );
    if( ++timer > (TIMEOUT / SPEED ))		// sleep after timeout
      stop = 0xFF;
  }
}
